from dataclasses import dataclass
from typing import List, TypedDict

from deltadefi.models.models import (
    AssetBalance,
    DepositRecord,
    OrderFillingRecordJSON,
    OrderJSON,
    WithdrawalRecord,
)


@dataclass
class CreateNewAPIKeyResponse(TypedDict):
    api_key: str


@dataclass
class GetOperationKeyResponse(TypedDict):
    encrypted_operation_key: str
    operation_key_hash: str


@dataclass
class BuildDepositTransactionResponse(TypedDict):
    tx_hex: str


@dataclass
class SubmitDepositTransactionResponse(TypedDict):
    tx_hash: str


@dataclass
class GetDepositRecordsResponse(List[DepositRecord]):
    pass


@dataclass
class GetWithdrawalRecordsResponse(List[WithdrawalRecord]):
    pass


@dataclass
class GetOrderRecordResponse(TypedDict):
    orders: List[OrderJSON]
    order_filling_records: List[OrderFillingRecordJSON]


@dataclass
class BuildWithdrawalTransactionResponse(TypedDict):
    tx_hex: str


@dataclass
class SubmitWithdrawalTransactionResponse(TypedDict):
    tx_hash: str


@dataclass
class GetAccountInfoResponse(TypedDict):
    api_key: str
    api_limit: int
    created_at: str
    updated_at: str


@dataclass
class GetAccountBalanceResponse(List[AssetBalance]):
    pass



from urllib.parse import urlencode

from deltadefi.error import (
    ParameterRequiredError,
    ParameterTypeError,
    ParameterValueError,
)


def clean_none_value(d) -> dict:
    out = {}
    for k in d.keys():
        if d[k] is not None:
            out[k] = d[k]
    return out


def check_required_parameter(value, name):
    if not value and value != 0:
        raise ParameterRequiredError([name])


def check_required_parameters(params):
    """Validate multiple parameters
    params = [
        ['btcusdt', 'symbol'],
        [10, 'price']
    ]

    """
    for p in params:
        check_required_parameter(p[0], p[1])


def check_enum_parameter(value, enum_class):
    if value not in set(item.value for item in enum_class):
        raise ParameterValueError([value])


def check_type_parameter(value, name, data_type):
    if value is not None and not isinstance(value, data_type):
        raise ParameterTypeError([name, data_type])


def encoded_string(query):
    return urlencode(query, True).replace("%40", "@")



#
from typing import List

from sidan_gin import Asset, UTxO

from deltadefi.api import API
from deltadefi.lib.utils import check_required_parameter, check_required_parameters
from deltadefi.models.models import OrderStatusType
from deltadefi.responses import (
    BuildDepositTransactionResponse,
    BuildWithdrawalTransactionResponse,
    CreateNewAPIKeyResponse,
    GetAccountBalanceResponse,
    GetDepositRecordsResponse,
    GetOrderRecordResponse,
    GetWithdrawalRecordsResponse,
    SubmitDepositTransactionResponse,
    SubmitWithdrawalTransactionResponse,
)
from deltadefi.responses.accounts import GetOperationKeyResponse


class Accounts(API):
    """
    Accounts client for interacting with the DeltaDeFi API.
    """

    group_url_path = "/accounts"

    def __init__(self, api_key=None, base_url=None, **kwargs):
        super().__init__(api_key=api_key, base_url=base_url, **kwargs)

    def get_operation_key(self, **kwargs) -> GetOperationKeyResponse:
        """
        Get the encrypted operation key.

        Returns:
            A GetOperationKeyResponse object containing the encrypted operation key and its hash.
        """

        url_path = "/operation-key"
        return self.send_request("GET", self.group_url_path + url_path, kwargs)

    def create_new_api_key(self, **kwargs) -> CreateNewAPIKeyResponse:
        """
        Create a new API key.

        Returns:
            A CreateNewAPIKeyResponse object containing the new API key.
        """

        url_path = "/new-api-key"
        return self.send_request("GET", self.group_url_path + url_path, kwargs)

    def get_deposit_records(self, **kwargs) -> GetDepositRecordsResponse:
        """
        Get deposit records.

        Returns:
            A GetDepositRecordsResponse object containing the deposit records.
        """
        url_path = "/deposit-records"
        return self.send_request("GET", self.group_url_path + url_path, kwargs)

    def get_withdrawal_records(self, **kwargs) -> GetWithdrawalRecordsResponse:
        """
        Get withdrawal records.

        Returns:
            A GetWithdrawalRecordsResponse object containing the withdrawal records.
        """
        url_path = "/withdrawal-records"
        return self.send_request("GET", self.group_url_path + url_path, kwargs)

    def get_order_records(
        self, status: OrderStatusType, **kwargs
    ) -> GetOrderRecordResponse:
        """
        Get order records.

        Args:
            status: The status of the order records to retrieve. It can be "openOrder",
                    "orderHistory", or "tradingHistory".
            limit: Optional; The maximum number of records to return. Defaults to 10, max 250.
            page: Optional; The page number for pagination. Defaults to 1.

        Returns:
            A GetOrderRecordResponse object containing the order records.
        """
        check_required_parameter(status, "status")
        payload = {"status": status, **kwargs}

        url_path = "/order-records"
        return self.send_request("GET", self.group_url_path + url_path, payload)

    def get_account_balance(self, **kwargs) -> GetAccountBalanceResponse:
        """
        Get account balance.

        Returns:
            A GetAccountBalanceResponse object containing the account balance.
        """
        url_path = "/balance"
        return self.send_request("GET", self.group_url_path + url_path, kwargs)

    def build_deposit_transaction(
        self, deposit_amount: List[Asset], input_utxos: List[UTxO], **kwargs
    ) -> BuildDepositTransactionResponse:
        """
        Build a deposit transaction.

        Args:
            data: A BuildDepositTransactionRequest object containing the deposit transaction details.

        Returns:
            A BuildDepositTransactionResponse object containing the built deposit transaction.
        """

        check_required_parameters(
            [[deposit_amount, "deposit_amount"], [input_utxos, "input_utxos"]]
        )
        payload = {
            "deposit_amount": deposit_amount,
            "input_utxos": input_utxos,
            **kwargs,
        }

        url_path = "/deposit/build"
        return self.send_request("POST", self.group_url_path + url_path, payload)

    def build_withdrawal_transaction(
        self, withdrawal_amount: List[Asset], **kwargs
    ) -> BuildWithdrawalTransactionResponse:
        """
        Build a withdrawal transaction.

        Args:
            data: A BuildWithdrawalTransactionRequest object containing the withdrawal transaction details.

        Returns:
            A BuildWithdrawalTransactionResponse object containing the built withdrawal transaction.
        """

        check_required_parameter(withdrawal_amount, "withdrawal_amount")
        payload = {"withdrawal_amount": withdrawal_amount, **kwargs}

        url_path = "/withdrawal/build"
        return self.send_request("POST", self.group_url_path + url_path, payload)

    def submit_deposit_transaction(
        self, signed_tx: str, **kwargs
    ) -> SubmitDepositTransactionResponse:
        """
        Submit a deposit transaction.

        Args:
            data: A SubmitDepositTransactionRequest object containing the deposit transaction details.

        Returns:
            A SubmitDepositTransactionResponse object containing the submitted deposit transaction.
        """

        check_required_parameter(signed_tx, "signed_tx")
        payload = {"signed_tx": signed_tx, **kwargs}

        url_path = "/deposit/submit"
        return self.send_request("POST", self.group_url_path + url_path, payload)

    def submit_withdrawal_transaction(
        self, signed_tx: str, **kwargs
    ) -> SubmitWithdrawalTransactionResponse:
        """
        Submit a withdrawal transaction.

        Args:
            data: A SubmitWithdrawalTransactionRequest object containing the withdrawal transaction details.

        Returns:
            A SubmitWithdrawalTransactionResponse object containing the submitted withdrawal transaction.
        """

        check_required_parameter(signed_tx, "signed_tx")
        payload = {"signed_tx": signed_tx, **kwargs}

        url_path = "/withdrawal/submit"
        return self.send_request("POST", self.group_url_path + url_path, payload)



import json

import requests

from deltadefi.error import ClientError, ServerError
from deltadefi.lib.utils import clean_none_value, encoded_string


class API(object):
    def __init__(self, base_url=None, api_key=None, timeout=None, **kwargs):
        self.base_url = base_url
        self.api_key = api_key
        self.timeout = timeout
        self.session = requests.Session()
        self.session.headers.update(
            {
                "Content-Type": "application/json;charset=utf-8",
                "X-API-KEY": api_key if api_key is not None else "",
            }
        )

    def send_request(self, http_method, url_path, payload=None):
        if payload is None:
            payload = {}
        url = self.base_url + url_path
        if http_method.upper() == "GET":
            params = clean_none_value(
                {
                    "url": url,
                    "params": self._prepare_params(payload),  # Query parameters
                    "timeout": self.timeout,
                }
            )
        else:
            params = clean_none_value(
                {
                    "url": url,
                    "json": payload,  # Use 'json' to send payload in the body
                    "timeout": self.timeout,
                }
            )

        response = self._dispatch_request(http_method)(**params)
        self._handle_exception(response)

        try:
            data = response.json()
        except ValueError:
            data = response.text
        result = {}

        if len(result) != 0:
            result["data"] = data
            return result

        return data

    def _dispatch_request(self, http_method):
        return {
            "GET": self.session.get,
            "DELETE": self.session.delete,
            "PUT": self.session.put,
            "POST": self.session.post,
        }.get(http_method, "GET")

    def _prepare_params(self, params):
        return encoded_string(clean_none_value(params))

    def _handle_exception(self, response):
        status_code = response.status_code
        if status_code < 400:
            return
        if 400 <= status_code < 500:
            try:
                err = json.loads(response.text)
            except json.JSONDecodeError:
                raise ClientError(
                    status_code, None, response.text, response.headers, None
                )
            error_data = None
            if "data" in err:
                error_data = err["data"]
            print("Error?", err)
            raise ClientError(
                status_code, err["code"], err["msg"], response.headers, error_data
            )
        raise ServerError(status_code, response.text)



# flake8: noqa: E501

from typing import Literal

from deltadefi.api import API
from deltadefi.lib.utils import check_required_parameter, check_required_parameters
from deltadefi.responses import (
    GetAggregatedPriceResponse,
    GetMarketDepthResponse,
    GetMarketPriceResponse,
)


class Market(API):
    """
    Markets client for interacting with the DeltaDeFi API.
    """

    group_url_path = "/market"

    def __init__(self, api_key=None, base_url=None, **kwargs):
        super().__init__(api_key=api_key, base_url=base_url, **kwargs)

    def get_depth(self, symbol: str, **kwargs) -> GetMarketDepthResponse:
        """
        Get market depth.

        Args:
            data: A GetMarketDepthRequest object containing the market pair.

        Returns:
            A GetMarketDepthResponse object containing the market depth.
        """

        check_required_parameter(symbol, "symbol")
        payload = {"symbol": symbol, **kwargs}
        url_path = "/depth"

        return self.send_request("GET", self.group_url_path + url_path, payload)

    def get_market_price(self, symbol: str, **kwargs) -> GetMarketPriceResponse:
        """
        Get market price.

        Args:
            data: A GetMarketPriceRequest object containing the market pair.

        Returns:
            A GetMarketPriceResponse object containing the market price.
        """
        check_required_parameter(symbol, "symbol")
        payload = {"symbol": symbol, **kwargs}
        url_path = "/market-price"
        return self.send_request("GET", self.group_url_path + url_path, payload)

    def get_aggregated_price(
        self,
        symbol: str,
        interval: Literal["15m", "30m", "1h", "1d", "1w", "1M"],
        start: int,
        end: int,
    ) -> GetAggregatedPriceResponse:
        """
        Get aggregated price.

        Args:
            data: A GetAggregatedPriceRequest object containing the market pair, interval, start, and end time.

        Returns:
            A GetAggregatedPriceResponse object containing the aggregated price.
        """

        check_required_parameters(
            [
                [symbol, "symbol"],
                [interval, "interval"],
                [start, "start"],
                [end, "end"],
            ]
        )
        url_path = f"/aggregated-trade/{symbol}"
        return self.send_request(
            "GET",
            self.group_url_path + url_path,
            {
                "interval": interval,
                "start": start,
                "end": end,
            },
        )



from deltadefi.api import API
from deltadefi.lib.utils import check_required_parameter, check_required_parameters
from deltadefi.models.models import OrderSide, OrderType
from deltadefi.responses import (
    BuildCancelOrderTransactionResponse,
    BuildPlaceOrderTransactionResponse,
    SubmitPlaceOrderTransactionResponse,
)


class Order(API):
    """
    Orders client for interacting with the DeltaDeFi API.
    """

    group_url_path = "/order"

    def __init__(self, api_key=None, base_url=None, **kwargs):
        super().__init__(api_key=api_key, base_url=base_url, **kwargs)

    def build_place_order_transaction(
        self,
        symbol: str,
        side: OrderSide,
        type: OrderType,
        quantity: int,
        **kwargs,
    ) -> BuildPlaceOrderTransactionResponse:
        """
        Build a place order transaction.

        Args:
            symbol: The trading pair symbol (e.g., "BTC-USD").
            side: The side of the order (e.g., "buy" or "sell").
            type: The type of the order (e.g., "limit" or "market").
            quantity: The quantity of the asset to be traded.
            **kwargs: Additional parameters for the order, such as price, limit_slippage, etc.

        Returns:
            A BuildPlaceOrderTransactionResponse object containing the built order transaction.
        """

        check_required_parameters(
            [
                [symbol, "symbol"],
                [side, "side"],
                [type, "type"],
                [quantity, "quantity"],
            ]
        )

        if type == "limit":
            check_required_parameter(kwargs.get("price"), "price")

        if type == "market" and kwargs.get("limit_slippage"):
            check_required_parameter(
                kwargs.get("max_slippage_basis_point"), "max_slippage_basis_point"
            )

        payload = {
            "symbol": symbol,
            "side": side,
            "type": type,
            "quantity": quantity,
            **kwargs,
        }

        url_path = "/build"
        return self.send_request("POST", self.group_url_path + url_path, payload)

    def build_cancel_order_transaction(
        self, order_id: str, **kwargs
    ) -> BuildCancelOrderTransactionResponse:
        """
        Build a cancel order transaction.

        Args:
            order_id: The ID of the order to be canceled.

        Returns:
            A BuildCancelOrderTransactionResponse object containing the built cancel order transaction.
        """

        check_required_parameter(order_id, "order_id")

        url_path = f"/{order_id}/build"
        return self.send_request("DELETE", self.group_url_path + url_path, **kwargs)

    def submit_place_order_transaction(
        self, order_id: str, signed_tx: str, **kwargs
    ) -> SubmitPlaceOrderTransactionResponse:
        """
        Submit a place order transaction.

        Args:
            data: A SubmitPlaceOrderTransactionRequest object containing the order details.

        Returns:
            A SubmitPlaceOrderTransactionResponse object containing the submitted order transaction.
        """
        check_required_parameters([[order_id, "order_id"], [signed_tx, "signed_tx"]])
        payload = {"order_id": order_id, "signed_tx": signed_tx, **kwargs}

        url_path = "/submit"
        return self.send_request("POST", self.group_url_path + url_path, payload)

    def submit_cancel_order_transaction(self, signed_tx: str, **kwargs):
        """
        Submit a cancel order transaction.

        Args:
            data: A SubmitCancelOrderTransactionRequest object containing the cancel order details.
        """
        check_required_parameter(signed_tx, "signed_tx")
        payload = {"signed_tx": signed_tx, **kwargs}

        path_url = "/submit"
        return self.send_request("DELETE", self.group_url_path + path_url, payload)



from dataclasses import dataclass
from typing import List, TypedDict

from deltadefi.models import OrderJSON


@dataclass
class GetTermsAndConditionResponse(TypedDict):
    value: str


@dataclass
class MarketDepth(TypedDict):
    price: float
    quantity: float


@dataclass
class GetMarketDepthResponse(TypedDict):
    bids: List[MarketDepth]
    asks: List[MarketDepth]


@dataclass
class GetMarketPriceResponse(TypedDict):
    price: float


@dataclass
class Trade(TypedDict):
    time: str
    symbol: str
    open: float
    high: float
    low: float
    close: float
    volume: float


@dataclass
class GetAggregatedPriceResponse(List[Trade]):
    pass


@dataclass
class BuildPlaceOrderTransactionResponse(TypedDict):
    order_id: str
    tx_hex: str


@dataclass
class SubmitPlaceOrderTransactionResponse(TypedDict):
    order: OrderJSON


@dataclass
class PostOrderResponse(SubmitPlaceOrderTransactionResponse):
    pass


@dataclass
class BuildCancelOrderTransactionResponse(TypedDict):
    tx_hex: str



from dataclasses import dataclass
from typing import Optional


@dataclass
class AuthHeaders:
    jwt: str
    apiKey: str


class ApiHeaders:
    __annotations__ = {
        "Content-Type": str,
        "Authorization": Optional[str],
        "X-API-KEY": Optional[str],
    }
